@@ -22,6 +22,8 @@
 """
 
 import math
+import warnings
+
 import numpy as np
 import bisect
 from abc import abstractmethod
@@ -32,7 +34,16 @@ import scipy
 import scipy.interpolate
 import cv2
 
-
+try:
+    import poly_calc
+except ModuleNotFoundError:
+    print("Cython module not found. Use pyximport temporally")
+    import pyximport
+    pyximport.install(language_level=3)  # '../elements/poly_calc.pyx',
+    import os
+    pyxroot = os.path.dirname(os.path.abspath(__file__))
+    poly_calc = pyximport.load_module('poly_calc', pyxroot+'\\poly_calc.pyx', language_level=3)
+    # import poly_calc
 
 
 ################## 基础的一维曲线类(抽象类) 现在已经弃用！#####################
@@ -119,6 +130,20 @@ class BasePolynomial(ExplicitCurve):
         else:
             self.valid_x_range = valid_x_range
 
+        self._lower_boundary_value = [None for _ in range(4)] # 分别对应valid_x_range上下界情况下的0-3阶导数
+        self._upper_boundary_value = [None for _ in range(4)]
+
+    def _get_boundary_value(self, order):
+        if (self._lower_boundary_value[order] is None) or (self._upper_boundary_value[order] is None):
+            x = np.ascontiguousarray(self.valid_x_range, dtype=np.float64)
+            coef = self.coef if order == 0 else self.derivative_coef(order)
+            c = np.ascontiguousarray(coef, dtype=np.float64)
+            val = poly_calc.evaluate(x, c, self.valid_x_range[0], self.valid_x_range[1], 0, 0.,0.,0.,0.)
+            val = np.asarray(val)
+            self._lower_boundary_value[order] = val[0] #self.evaluate(self.valid_x_range[0],order)
+            self._upper_boundary_value[order] = val[1] #self.evaluate(self.valid_x_range[1],order)
+        return self._lower_boundary_value[order], self._upper_boundary_value[order]
+
     def _out_of_range_flag(self, x) -> Union[bool, np.ndarray]:
         '''
         if x is a scalar, return bool
@@ -155,7 +180,7 @@ class BasePolynomial(ExplicitCurve):
 
 
     def set_coef(self, coef):
-        self.coef = np.array(coef)
+        self.coef = np.ascontiguousarray(coef, dtype=np.float64)
         self.order = self.coef.shape[0] - 1
         for i in range(1, 4):  # 先求1-3阶导数
             self.derivative_coef(order=i)
@@ -171,40 +196,92 @@ class BasePolynomial(ExplicitCurve):
             # if order >= len(self.coef):
             dcoef = np.polyder(self.coef, order)
             # if order >= len(self.coef), dcoef is array([]), and polyval([],x) === 0
-            self._derivative_coefs[order] = dcoef
+            self._derivative_coefs[order] = np.ascontiguousarray(dcoef, dtype=np.float64)
             return dcoef
 
 
     def calc_point(self, x):
-        x = np.array(x, dtype=np.float)
-        extra_mask = self._out_of_range_flag(x)
-        val = np.empty_like(x)
-        val[extra_mask] = self.extrapolate(x[extra_mask], order=0) # 数据范围外，按外推规则计算
-        val[~extra_mask] = np.polyval(self.coef, x[~extra_mask]) # 数据范围内，直接计算
+        y_0, y_end = self._get_boundary_value(0)
+        y_prime_0, y_prime_end = self._get_boundary_value(1)
+        c = np.ascontiguousarray(self.coef, dtype=np.float64)
+
+        if self._isscalar(x):
+            return poly_calc.evaluate_scalar(x, c, self.valid_x_range[0], self.valid_x_range[1], 0,
+                                             y_0, y_prime_0, y_end, y_prime_end)
+
+        x = np.ascontiguousarray(x, dtype=np.float64)
+        val = poly_calc.evaluate(x, c, self.valid_x_range[0], self.valid_x_range[1], 0,
+                                 y_0, y_prime_0, y_end, y_prime_end)
+        val = np.asarray(val)
+
+        # x = np.asarray(x, dtype=float)
+        # extra_mask = self._out_of_range_flag(x)
+        # val = np.empty_like(x)
+        # val[extra_mask] = self.extrapolate(x[extra_mask], order=0) # 数据范围外，按外推规则计算
+        # val[~extra_mask] = np.polyval(self.coef, x[~extra_mask]) # 数据范围内，直接计算
         return val
 
     def calc_first_derivative(self,x):
-        x = np.array(x, dtype=np.float)
-        extra_mask = self._out_of_range_flag(x)
-        val = np.empty_like(x)
-        val[extra_mask] = self.extrapolate(x[extra_mask], order=1)  # 数据范围外，按外推规则计算
-        val[~extra_mask] = np.polyval(self.derivative_coef(order=1), x[~extra_mask]) # 数据范围内，直接计算
+        y_0, y_end = self._get_boundary_value(0)
+        y_prime_0, y_prime_end = self._get_boundary_value(1)
+        c = np.ascontiguousarray(self.derivative_coef(order=1), dtype=np.float64)
+
+        if self._isscalar(x):
+            return poly_calc.evaluate_scalar(x, c, self.valid_x_range[0], self.valid_x_range[1], 1, y_0, y_prime_0, y_end,
+                                      y_prime_end)
+
+        x = np.ascontiguousarray(x, dtype=np.float64)
+        val = poly_calc.evaluate(x, c, self.valid_x_range[0], self.valid_x_range[1], 1,
+                                 y_0, y_prime_0, y_end, y_prime_end)
+        val = np.asarray(val)
+
+        # x = np.asarray(x, dtype=float)
+        # extra_mask = self._out_of_range_flag(x)
+        # val = np.empty_like(x)
+        # val[extra_mask] = self.extrapolate(x[extra_mask], order=1)  # 数据范围外，按外推规则计算
+        # val[~extra_mask] = np.polyval(self.derivative_coef(order=1), x[~extra_mask]) # 数据范围内，直接计算
         return val
 
     def calc_second_derivative(self,x):
-        x = np.array(x, dtype=np.float)
-        extra_mask = self._out_of_range_flag(x)
-        val = np.empty_like(x)
-        val[extra_mask] = self.extrapolate(x[extra_mask], order=2)  # 数据范围外，按外推规则计算
-        val[~extra_mask] = np.polyval(self.derivative_coef(order=2), x[~extra_mask])  # 数据范围内，直接计算
+        y_0, y_end = self._get_boundary_value(0)
+        y_prime_0, y_prime_end = self._get_boundary_value(1)
+        c = np.ascontiguousarray(self.derivative_coef(order=2), dtype=np.float64)
+
+        if self._isscalar(x):
+            return poly_calc.evaluate_scalar(x, c, self.valid_x_range[0], self.valid_x_range[1], 2, y_0, y_prime_0, y_end,
+                                      y_prime_end)
+
+        x = np.ascontiguousarray(x, dtype=np.float64)
+        val = poly_calc.evaluate(x, c, self.valid_x_range[0], self.valid_x_range[1], 2,
+                                 y_0, y_prime_0, y_end, y_prime_end)
+        val = np.asarray(val)
+
+        # x = np.asarray(x, dtype=float)
+        # extra_mask = self._out_of_range_flag(x)
+        # val = np.empty_like(x)
+        # val[extra_mask] = self.extrapolate(x[extra_mask], order=2)  # 数据范围外，按外推规则计算
+        # val[~extra_mask] = np.polyval(self.derivative_coef(order=2), x[~extra_mask])  # 数据范围内，直接计算
         return val
 
     def calc_third_derivative(self, x):
-        x = np.array(x, dtype=np.float)
-        extra_mask = self._out_of_range_flag(x)
-        val = np.empty_like(x)
-        val[extra_mask] = self.extrapolate(x[extra_mask], order=3)  # 数据范围外，按外推规则计算
-        val[~extra_mask] = np.polyval(self.derivative_coef(order=3), x[~extra_mask])  # 数据范围内，直接计算
+        y_0, y_end = self._get_boundary_value(0)
+        y_prime_0, y_prime_end = self._get_boundary_value(1)
+        c = np.ascontiguousarray(self.derivative_coef(order=3), dtype=np.float64)
+
+        if self._isscalar(x):
+            return poly_calc.evaluate_scalar(x, c, self.valid_x_range[0], self.valid_x_range[1], 3, y_0, y_prime_0, y_end,
+                                      y_prime_end)
+
+        x = np.ascontiguousarray(x, dtype=np.float64)
+        val = poly_calc.evaluate(x, c, self.valid_x_range[0], self.valid_x_range[1], 3,
+                                 y_0, y_prime_0, y_end, y_prime_end)
+        val = np.asarray(val)
+
+        # x = np.asarray(x, dtype=float)
+        # extra_mask = self._out_of_range_flag(x)
+        # val = np.empty_like(x)
+        # val[extra_mask] = self.extrapolate(x[extra_mask], order=3)  # 数据范围外，按外推规则计算
+        # val[~extra_mask] = np.polyval(self.derivative_coef(order=3), x[~extra_mask])  # 数据范围内，直接计算
         return val
 
     def fit(self, x_data, y_data, order):
@@ -357,8 +434,11 @@ class QuinticPolynomial(BasePolynomial):
         '''
         interpolate with two_point_boundary_value constraints.
         And it is recommended to set x_0 to 0 to accelerate the calculation
+        What's more, a large number (like x>1e5) could cause Ill-Conditioned Matrix!
         todo: 可以提前储存几次方是几，可以加速
         '''
+        if x_0 > 1e5: warnings.warn("WARNING! A large number (like x>1e5) could cause ill-Conditioned Matrix!")
+
         self.valid_x_range = [x_0, x_end]
 
         if x_0 != 0:
@@ -424,7 +504,7 @@ class PiecewiseQuinticPolynomial(ExplicitCurve):
             [xn, yn, y_prime_n, y_2prime_n]
         ])
         '''
-        all_points_with_derivatives = np.array(all_points_with_derivatives)
+        all_points_with_derivatives = np.asarray(all_points_with_derivatives)
         if self.segment_num is None:
             self.segment_num = len(all_points_with_derivatives) - 1
             self.segments: List[QuinticPolynomial] = [QuinticPolynomial() for _ in range(self.segment_num)]
@@ -538,7 +618,7 @@ class InterpolationCurve(ExplicitCurve):
 
     def calc_point(self, x):
         # val = self.interpolate(x, order=0)
-        x = np.array(x, dtype=np.float)
+        x = np.asarray(x, dtype=float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=0)  # 数据范围外，按外推规则计算
@@ -547,7 +627,7 @@ class InterpolationCurve(ExplicitCurve):
         return val
 
     def calc_first_derivative(self, x):
-        x = np.array(x, dtype=np.float)
+        x = np.asarray(x, dtype=float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=1)  # 数据范围外，按外推规则计算
@@ -555,7 +635,7 @@ class InterpolationCurve(ExplicitCurve):
         return val
 
     def calc_second_derivative(self, x):
-        x = np.array(x, dtype=np.float)
+        x = np.asarray(x, dtype=float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=2)  # 数据范围外，按外推规则计算
@@ -563,7 +643,7 @@ class InterpolationCurve(ExplicitCurve):
         return val
 
     def calc_third_derivative(self, x):
-        x = np.array(x, dtype=np.float)
+        x = np.asarray(x, dtype=float)
         extra_mask = self._out_of_range_flag(x)
         val = np.empty_like(x)
         val[extra_mask] = self.extrapolate(x[extra_mask], order=3)  # 数据范围外，按外推规则计算
@@ -575,17 +655,23 @@ class spCubicSpline(InterpolationCurve):
     """
     CubicSpline class
     三次样条插值
-    todo: scipy自带的在距离很远的时候插值有严重问题，暂时先用自己写的，以后看看为什么有这个问题并且怎么修复
     """
-    def __init__(self, x=None, y=None):
+    def __init__(self, x=None, y=None, bc_type='natural'):
         self._sp_csp = None # 屈服了，还是用Scipy的吧
+        self.bc_type = bc_type
         super(spCubicSpline, self).__init__(x, y)
 
     def _calc_coef(self):
-        self._sp_csp = scipy.interpolate.CubicSpline(self.x, self.y)
+        self._sp_csp = scipy.interpolate.CubicSpline(self.x, self.y, bc_type=self.bc_type)#bc_type=((1, 0), (1, 0)))
+        #，bc_type=((1, 0), (1, 0)) 指定了第一个和最后一个插值点处的一阶导数为0
+        # (1, 0) 表示一阶导数为0，(2, 0) 表示二阶导数为0
+        # 不加这个限制的话会出现非常大的振荡,，因为默认Not-a-knot Spline (非结点样条)
+        # Natural Spline (自然样条): 在这种情况下，样条的二阶导数在两个端点都被设定为零。这是默认的边界条件。
+        # Clamped Spline (夹持样条): 在这种情况下，你可以指定首尾两个点的一阶导数值。这相当于夹持（固定）样条的两个端点。
+        # Not-a-knot Spline (非结点样条): 在这种情况下，样条的三阶导数在内部节点上是连续的。这种条件通常在要求样条在内部节点处更平滑的情况下使用。
 
     def interpolate(self, x, order):
-        if np.array(x).size == 0:
+        if np.asarray(x).size == 0:
             return np.empty_like(x)
 
         val = self._sp_csp(x, order)
@@ -692,7 +778,7 @@ class myCubicSpline(InterpolationCurve):
             return seg_idxs #如果是最后一个点，那么segment id要往前挪一个
 
     def interpolate(self, x, order):
-        if np.array(x).size == 0:
+        if np.asarray(x).size == 0:
             return np.empty_like(x)
 
         i = self._search_index(x)
@@ -747,7 +833,7 @@ class myCubicSpline(InterpolationCurve):
         return val
 
 
-class CubicSpline(myCubicSpline):
+class CubicSpline(spCubicSpline):
     pass
 
 class Spline:#Cubic CubicSpline class(abandoned)
@@ -1037,7 +1123,7 @@ class BezierCurve(ParametricCurve):
         super(BezierCurve, self).__init__()
         assert len(control_points) >= 2
         self.n = len(control_points) - 1 # 贝塞尔曲线的阶数
-        self.control_points = np.array(control_points)
+        self.control_points = np.asarray(control_points)
 
         # self._bezier_coeff_dict = {}
         # self._binom_coeff_dict = {}
@@ -1109,7 +1195,7 @@ class BezierCurve(ParametricCurve):
             num_pts = len(t)
             point = np.zeros((num_pts, 2), dtype=float)
 
-        t = np.array(t)
+        t = np.asarray(t)
         assert np.all(t <= 1) and np.all(t >= 0)
 
         for i in range(self.n+1):
@@ -1154,16 +1240,16 @@ class BezierCurve(ParametricCurve):
             return self.derivative_bezier_curve.calc_second_derivative_t(t)
 
     def calc_point(self, s):
-        return self.calc_point_t( np.array(s) / self.arclength )
+        return self.calc_point_t( np.asarray(s) / self.arclength )
 
     def calc_first_derivative(self, s):
-        return self.calc_first_derivative_t( np.array(s) / self.arclength )
+        return self.calc_first_derivative_t( np.asarray(s) / self.arclength )
 
     def calc_second_derivative(self, s):
-        return self.calc_second_derivative_t( np.array(s) / self.arclength )
+        return self.calc_second_derivative_t( np.asarray(s) / self.arclength )
 
     def calc_third_derivative(self, s):
-        return self.calc_third_derivative_t( np.array(s) / self.arclength )
+        return self.calc_third_derivative_t( np.asarray(s) / self.arclength )
 
 
 
